from datetime import datetime
from os import getenv
from typing import Optional

from aars import AARS
from aleph.sdk.client import AuthenticatedAlephClient
from aleph.sdk.chains.sol import get_fallback_account
from aleph.sdk.conf import settings
from aleph.sdk.types import Account
from aleph.sdk.vm.cache import TestVmCache, VmCache

from .constants import SERVICE_MARKETS_MESSAGE_CHANNEL, SERVICE_MARKETS_MANAGER_PUBKEYS


async def initialize_aars(
    test_cache_flag: Optional[bool] = None,
    test_channel_flag: Optional[bool] = None,
    custom_channel: Optional[str] = None,
    account: Optional[Account] = None,
    aleph_session: Optional[AuthenticatedAlephClient] = None,
) -> AARS:
    test_cache_flag = test_cache_flag or getenv("TEST_CACHE")
    if test_cache_flag is not None and test_cache_flag.lower() == "false":
        cache = VmCache()
    else:
        cache = TestVmCache()

    aleph_account = get_fallback_account() if account is None else account
    aleph_session = (
        AuthenticatedAlephClient(aleph_account, settings.API_HOST)
        if aleph_session is None
        else aleph_session
    )

    test_channel_flag = test_channel_flag or getenv("TEST_CHANNEL")
    custom_channel = (
        getenv("CUSTOM_CHANNEL") if custom_channel is None else custom_channel
    )
    if custom_channel:
        channel = custom_channel
    elif test_channel_flag is not None and test_channel_flag.lower() == "true":
        channel = "SERVICE_MARKETS_TEST_" + str(datetime.now())
    else:
        channel = SERVICE_MARKETS_MESSAGE_CHANNEL

    if not channel:
        channel = SERVICE_MARKETS_MESSAGE_CHANNEL

    print("Using channel: " + channel)

    aars = AARS(
        account=aleph_account, channel=channel, cache=cache, session=aleph_session
    )
    print(f"Using account: {aleph_account.get_address()}")
    if aleph_account.get_address() in SERVICE_MARKETS_MANAGER_PUBKEYS:
        try:
            resp, status = await aleph_session.fetch_aggregate(
                "security", aleph_account.get_address()
            )
            existing_authorizations = resp.json().get("authorizations", [])
        except:
            existing_authorizations = []
        needed_authorizations = [
            {
                "address": address,
                "channels": [SERVICE_MARKETS_MESSAGE_CHANNEL],
            }
            for address in SERVICE_MARKETS_MANAGER_PUBKEYS
        ]
        # if not all(auth in existing_authorizations for auth in needed_authorizations):
        #    aggregate = {
        #        "authorizations": needed_authorizations,
        #    }
        await aleph_session.create_aggregate(
            "security",
            {"authorizations": needed_authorizations},
            aleph_account.get_address(),
            channel="security",
        )

    return aars



from typing import List, Optional

from aars import Index
from pydantic import BaseModel

from ..core.model import (
    Permission,
    UserInfo,
    Service,
    Vote,
    Comment, Payment,
)


Index(Service, "owner_address")

Index(Permission, "user_address")
Index(Permission, "service_id")
Index(Permission, ["user_address", "service_id"])

Index(Payment, "txHash")

Index(Comment, "service_id")

Index(Vote, ["item_id", "user_address"])

Index(UserInfo, "address")
Index(UserInfo, "username")


class UploadServiceRequest(BaseModel):
    item_hash: Optional[str]
    name: str
    description: str
    url: str
    image_url: Optional[str]
    price: float
    owner_address: str
    tags: List[str] = []


class ServiceWithPermissionStatus(Service):
    permitted: Optional[bool] = None


class VoteServiceResponse(BaseModel):
    vote: Vote
    service: Service


class PutInvoiceServiceResponse(BaseModel):
    service: Service
    payment: Payment
    permission: Permission


class VoteCommentResponse(BaseModel):
    vote: Vote
    service: Comment


class PutUserInfo(BaseModel):
    username: str
    address: str
    bio: Optional[str]
    email: Optional[str]
    link: Optional[str]


class MessageResponse(BaseModel):
    response: str



from typing import List, Optional

from fastapi import APIRouter, HTTPException
from fastapi_walletauth import WalletAuthDep

from ...core.model import Permission, UserInfo
from ..api_model import PutUserInfo

router = APIRouter(
    prefix="/users",
    tags=["users"],
    responses={404: {"description": "Not found"}},
)


@router.get("")
async def get_users(
    username: Optional[str] = None,
    address: Optional[str] = None,
    page: int = 1,
    page_size: int = 20,
) -> List[UserInfo]:
    params = {}
    if username:
        params["username"] = username
    if address:
        params["address"] = address
    if params:
        return await UserInfo.filter(**params).page(page=page, page_size=page_size)
    return await UserInfo.fetch_objects().page(page=page, page_size=page_size)


@router.put("")
async def put_user_info(user_info: PutUserInfo, wallet: WalletAuthDep) -> UserInfo:
    user_record = None
    if user_info.address != wallet.address:
        raise HTTPException(
            status_code=403,
            detail="address does not match currently authorized user wallet",
        )
    if user_info.address:
        user_record = await UserInfo.filter(address=user_info.address).first()
        if user_record:
            user_record.username = user_info.username
            user_record.address = user_info.address
            user_record.bio = user_info.bio
            user_record.email = user_info.email
            user_record.link = user_info.link
            await user_record.save()
    if user_record is None:
        user_record = await UserInfo(
            username=user_info.username,
            address=user_info.address,
            bio=user_info.bio,
            email=user_info.email,
            link=user_info.link,
        ).save()
    return user_record


@router.get("/{address}")
async def get_specific_user(address: str) -> Optional[UserInfo]:
    return await UserInfo.filter(address=address).first()


@router.get("/{address}/permissions")
async def get_permissions(
    address: str,
    page: int = 1,
    page_size: int = 20,
) -> List[Permission]:
    permission_records = await Permission.filter(user_address=address).page(
        page=page, page_size=page_size
    )
    return permission_records



import aiohttp
import requests
import json

from .model import Payment


async def fetch_payment(tx_hash: str) -> Payment:
    url = 'https://api.thegraph.com/subgraphs/name/requestnetwork/request-payments-goerli'

    headers = {'Content-Type': 'application/json'}
    request_json = json.dumps({"query": payment_query.format(tx_hash)})

    # Initiate the session
    async with aiohttp.ClientSession() as session:
        # Post the query
        async with session.post(url, headers=headers, data=request_json) as response:
            # Raise an exception in case of status error
            response.raise_for_status()

            # Fetch the json response
            json_response = await response.json()

    return Payment(**(await response.json())['data']['payments'][0])


def fetch_payment_sync(tx_hash: str) -> Payment:
    # The endpoint URL for the Request Payments Subgraph
    url = 'https://api.thegraph.com/subgraphs/name/requestnetwork/request-payments-goerli'

    headers = {'Content-Type': 'application/json'}

    request_json = json.dumps({"query": payment_query.format(tx_hash)})

    # Post the query
    response = requests.post(url, headers=headers, data=request_json)

    # Raise an exception in case of status error
    response.raise_for_status()

    return Payment(**response.json()['data']['payments'][0])


payment_query = """
{{
  payments(where: {{txHash: "{0}"}}, first: 1) {{
    amount
    txHash
    from
    to
    contractAddress
    tokenAddress
    reference
  }}
}}
"""
